@@ -129,19 +129,34 @@ export class NotebookHandler {
 
         this._scenesChanged();
     }
-    toggleSceneMembershipOfCurrentCell() {
-        const cell = this._nbTracker.activeCell;
-        if(!cell) return;
+    toggleSceneMembershipOfSelectedCells() {
+        if(!this._nbTracker.currentWidget) return;
+        if(!this._nbTracker.activeCell) return;
 
         const current_scene = this._sceneDB.getActiveScene();
+        const is_init_scene = current_scene == this.getInitScene();
         const tag = 'scene__' + current_scene;
-        
-        if(!cell.model.metadata.get(tag)) {
-            cell.model.metadata.set(tag, true);
-        } else {
-            cell.model.metadata.delete(tag);
-        }
-        this._updateCellClassAndTags(cell, tag);
+        const notebook = this._nbTracker.currentWidget.content;
+
+        const set_membership = !this._nbTracker.activeCell.model.metadata.get(tag);
+
+        notebook.widgets.forEach((cell: Cell) => {
+            if(!notebook.isSelectedOrActive(cell)) return;
+            if(cell.model.type != 'code') return;
+
+            if(set_membership) {
+                cell.model.metadata.set(tag, true);
+                if(is_init_scene) {
+                    cell.model.metadata.set('init_cell', true);
+                }
+            } else {
+                cell.model.metadata.delete(tag);
+                if(is_init_scene) {
+                    cell.model.metadata.delete('init_cell');
+                }
+            }
+            this._updateCellClassAndTags(cell, tag);
+        });
     }
 
     // **** scene management and running *****************************************
@@ -196,12 +211,20 @@ export class NotebookHandler {
     
     // **** various **************************************************************
 
-    updateCellClassesAndTags(notebook: Notebook, scene_name: string) {
+    updateCellClassesAndTags(notebook: Notebook, scene_name:(string|null)=null, cell:(Cell|null)=null) {
         // console.log('updating', scene_name)
+
+        if(scene_name == null) scene_name = this.getActiveScene()!;
         const scene_tag = this._getSceneTag(scene_name);
-        notebook.widgets.map((cell: Cell) => {
-            this._updateCellClassAndTags(cell, scene_tag)
-        });
+
+        if(cell == null) {
+            notebook.widgets.map((cell: Cell) => {
+                this._updateCellClassAndTags(cell, scene_tag);
+            });
+        } else {
+            this._updateCellClassAndTags(cell, scene_tag);
+        }
+
     }
     jumpToNextSceneCell() {
         const presentCell = this._nbTracker.activeCell;
@@ -236,11 +259,50 @@ export class NotebookHandler {
             }
         }
     }
+    importLegacyInitializationCells(notebook: Notebook) {
+
+        let init_scenes_consistent = true;
+        let legacy_init_cells_exist = false;
+        let init_scene = this.getInitScene();
+        let init_scene_tag = (init_scene != null) ? this._getSceneTag(init_scene) : null;
+
+        // find out if there are legacy init cells and, if so, whether they are consistent with the scenes init cell
+        notebook.widgets.map((cell: Cell) => {
+            let is_legacy_init_cell = !!cell.model.metadata.get('init_cell');
+            let is_scenes_init_cell = init_scene_tag != null && !!cell.model.metadata.get(init_scene_tag);
+            if(is_legacy_init_cell) {
+                legacy_init_cells_exist = true;
+            }
+            if(is_legacy_init_cell != is_scenes_init_cell) {
+                init_scenes_consistent = false;
+            }
+        }); 
+        
+        if(!init_scenes_consistent && legacy_init_cells_exist) {
+            const scene_name = 'Legacy Init';
+            
+            notebook.widgets.map((cell: Cell) => {
+                let is_legacy_init_cell = !!cell.model.metadata.get('init_cell');
+                if(is_legacy_init_cell) {
+                    cell.model.metadata.set(this._getSceneTag(scene_name), true);
+                }
+            });
+
+            const scene_list = this.getScenesList();
+            if(!scene_list.includes(scene_name)) {
+                scene_list.push(scene_name)
+                this._sceneDB.setScenesList(scene_list);
+            }
+            this.toggleInitScene(scene_name);
+            this.setActiveScene(scene_name);
+        }
+    }
+
 
     /* ****************************************************************************************************************************************
      * Various private helper methods
      * ****************************************************************************************************************************************/
-    
+
     private _updateCellClassAndTags(cell: Cell, scene_tag: string) {
         let cell_tags: string[] = [];
         if(cell.model.metadata.has("tags")) {
@@ -263,6 +325,21 @@ export class NotebookHandler {
 
     }
 
+    private _writeCellMetadataForLegacyInitializationCellsPlugin(notebook: Notebook) {
+
+        let init_scene = this.getInitScene();
+        let init_scene_tag = (init_scene != null) ? this._getSceneTag(init_scene) : null;
+
+        notebook.widgets.map((cell: Cell) => {
+            if(init_scene_tag != null && !!cell.model.metadata.get(init_scene_tag)) {
+                cell.model.metadata.set('init_cell', true);
+            } else {
+                cell.model.metadata.delete('init_cell');
+            }
+        });
+    }
+
+    
     private _activateCellAndExpandParentHeadings(cell: Cell) {
         NotebookActions.expandParent(cell, this._nbTracker.currentWidget!.content);
         cell.activate();
@@ -328,6 +405,8 @@ export class NotebookHandler {
             }
         });
 
+        this._writeCellMetadataForLegacyInitializationCellsPlugin(activeNotebookPanel.content);
+
         this.scenesChanged.emit(void 0);
     }
 
